@@ -1,13 +1,26 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * e-table-sort-info.c: a Table Model
+/* 
+ * e-table-sort-info.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@gnu.org)
- *   Chris Lahey (clahey@ximian.com)
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
- * (C) 1999, 2000, 2001 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 
 #include "e-table-sort-info.h"
@@ -96,7 +109,7 @@ e_table_sort_info_class_init (ETableSortInfoClass *klass)
 }
 
 E_MAKE_TYPE(e_table_sort_info, "ETableSortInfo", ETableSortInfo,
-	    e_table_sort_info_class_init, e_table_sort_info_init, PARENT_TYPE);
+	    e_table_sort_info_class_init, e_table_sort_info_init, PARENT_TYPE)
 
 static void
 e_table_sort_info_sort_info_changed (ETableSortInfo *info)
